@@ -1,4 +1,4 @@
-/*      $Id: hw_irman.c,v 5.11 2009/04/10 16:03:08 lirc Exp $      */
+/*      $Id: hw_irman.c,v 5.13 2010/04/11 18:50:38 lirc Exp $      */
 
 /****************************************************************************
  ** hw_irman.c **********************************************************
@@ -14,6 +14,10 @@
 # include <config.h>
 #endif
 
+#ifndef LIRC_IRTTY
+#define LIRC_IRTTY "/dev/ttyS0"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -44,14 +48,13 @@ ir_code code;
 
 struct hardware hw_irman=
 {
-	LIRC_DRIVER_DEVICE,       /* default device */
+	LIRC_IRTTY,               /* default device */
 	-1,                       /* fd */
 	LIRC_CAN_REC_LIRCCODE,    /* features */
 	0,                        /* send_mode */
 	LIRC_MODE_LIRCCODE,       /* rec_mode */
 	CODE_LENGTH,              /* code_length */
 	irman_init,               /* init_func */
-	NULL,                     /* config_func */
 	irman_deinit,             /* deinit_func */
 	NULL,                     /* send_func */
 	irman_rec,                /* rec_func */
