@@ -1,4 +1,4 @@
-/*      $Id: hw_pixelview.c,v 5.15 2005/07/10 08:34:12 lirc Exp $      */
+/*      $Id: hw_pixelview.c,v 5.18 2010/04/11 18:50:38 lirc Exp $      */
 
 /****************************************************************************
  ** hw_pixelview.c **********************************************************
@@ -14,6 +14,10 @@
 # include <config.h>
 #endif
 
+#ifndef LIRC_IRTTY
+#define LIRC_IRTTY "/dev/ttyS0"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
@@ -32,21 +36,20 @@
 
 extern struct ir_remote *repeat_remote,*last_remote;
 
-unsigned char b[3];
-struct timeval start,end,last;
-lirc_t gap,signal_length;
-ir_code pre,code;
+static unsigned char b[3];
+static struct timeval start,end,last;
+static lirc_t signal_length;
+static ir_code pre,code;
 
 struct hardware hw_pixelview=
 {
-	LIRC_DRIVER_DEVICE,       /* default device */
+	LIRC_IRTTY,               /* default device */
 	-1,                       /* fd */
 	LIRC_CAN_REC_LIRCCODE,    /* features */
 	0,                        /* send_mode */
 	LIRC_MODE_LIRCCODE,       /* rec_mode */
 	30,                       /* code_length */
 	pixelview_init,           /* init_func */
-	NULL,                     /* config_func */
 	pixelview_deinit,         /* deinit_func */
 	NULL,                     /* send_func */
 	pixelview_rec,            /* rec_func */
@@ -58,7 +61,9 @@ struct hardware hw_pixelview=
 
 int pixelview_decode(struct ir_remote *remote,
 		     ir_code *prep,ir_code *codep,ir_code *postp,
-		     int *repeat_flagp,lirc_t *remaining_gapp)
+		     int *repeat_flagp,
+		     lirc_t *min_remaining_gapp,
+		     lirc_t *max_remaining_gapp)
 {
 #if 0
 	if(remote->pone!=0  ||  remote->sone!=833) return(0);
@@ -71,35 +76,9 @@ int pixelview_decode(struct ir_remote *remote,
 		return(0);
 	}
 
-	gap=0;
-	if(start.tv_sec-last.tv_sec>=2) /* >1 sec */
-	{
-		*repeat_flagp=0;
-	}
-	else
-	{
-		gap=(start.tv_sec-last.tv_sec)*1000000+
-		start.tv_usec-last.tv_usec;
-		
-		if(gap<remote->remaining_gap*(100+remote->eps)/100
-		   || gap<=remote->remaining_gap+remote->aeps)
-			*repeat_flagp=1;
-		else
-			*repeat_flagp=0;
-	}
+	map_gap(remote, &start, &last, signal_length, repeat_flagp,
+		min_remaining_gapp, max_remaining_gapp);
 	
-	*remaining_gapp=is_const(remote) ?
-	(remote->gap>signal_length ? remote->gap-signal_length:0):
-	remote->gap;
-
-	LOGPRINTF(1,"pre: %llx",(unsigned long long) *prep);
-	LOGPRINTF(1,"code: %llx",(unsigned long long) *codep);
-	LOGPRINTF(1,"repeat_flag: %d",*repeat_flagp);
-	LOGPRINTF(1,"gap: %lu",(unsigned long) gap);
-	LOGPRINTF(1,"rem: %lu",(unsigned long) remote->remaining_gap);
-	LOGPRINTF(1,"signal length: %lu",(unsigned long) signal_length);
-
-
 	return(1);
 }
 
