@@ -1,4 +1,4 @@
-/*      $Id: hw_slinke.c,v 5.7 2001/12/08 15:07:03 lirc Exp $      */
+/*      $Id: hw_slinke.c,v 5.12 2010/04/11 18:50:38 lirc Exp $      */
 
 /****************************************************************************
  ** hw_slinke.c ***********************************************************
@@ -22,6 +22,10 @@
 # include <config.h>
 #endif
 
+#ifndef LIRC_IRTTY
+#define LIRC_IRTTY "/dev/ttyS0"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
@@ -71,7 +75,7 @@ struct port_queue_rec{
     int length,bufsize;
     unsigned char *buf;
 };
-struct port_queue_rec queue[MAX_PORT_COUNT];
+static struct port_queue_rec queue[MAX_PORT_COUNT];
 
 /* values read from Slink-e*/
 
@@ -80,30 +84,28 @@ struct slinke_settings_rec{
    int  timeout_samples; /* number of sample periods */
    char *version;
 };
-struct slinke_settings_rec slinke_settings = {0,0,NULL};
+static struct slinke_settings_rec slinke_settings = {0,0,NULL};
 
 extern struct ir_remote *repeat_remote,*last_remote;
 
-struct timeval start,end,last;
-lirc_t gap,signal_length;
-ir_code pre,code;
-
-lirc_t slinke_readdata(void);
+static struct timeval start,end,last;
+static lirc_t signal_length;
+static ir_code pre,code;
 
 struct hardware hw_slinke = {
-    LIRC_DRIVER_DEVICE, /* default device */
+    LIRC_IRTTY,         /* default device */
     -1,                 /* fd */
     LIRC_CAN_REC_MODE2, /* features */
     0,                  /* send_mode */
     LIRC_MODE_MODE2,    /* rec_mode */
     0,                  /* code_length */
     slinke_init,        /* init_func */
-    NULL,               /* config_func */
     slinke_deinit,      /* deinit_func */
     NULL,               /* send_func */
     slinke_rec,         /* rec_func */
     slinke_decode,      /* decode_func */
-    slinke_readdata,	/* readdata */
+    NULL,               /* ioctl_func */
+    slinke_readdata,    /* readdata */
     "slinke"
 };
 
@@ -417,8 +419,9 @@ char *slinkePorts[] = {"SL0","SL1","SL2","SL3","IR0","PAR","SER","SYS"};
 #endif
 
 /*****************************************************************************/
-lirc_t slinke_readdata(void){
+lirc_t slinke_readdata(int timeout){
     lirc_t result;
+
     if (signal_queue_buf == NULL) return 0;
     if (signal_queue_rd_idx < signal_queue_length){
        result = signal_queue_buf[signal_queue_rd_idx++];
@@ -744,20 +747,22 @@ char *slinke_rec(struct ir_remote *remotes){
 /*****************************************************************************/
 extern struct rbuf rec_buffer;
 
-int slinke_decode(struct ir_remote *remote
-                 ,ir_code          *prep
-                 ,ir_code          *codep
-                 ,ir_code          *postp
-                 ,int              *repeat_flagp
-                 ,lirc_t           *remaining_gapp){
-
-    rewind_rec_buffer(); 
-    rec_buffer.wptr = 0; 
-    signal_queue_rd_idx = 0; 
-	return receive_decode(remote
-                         ,prep
-                         ,codep
-                         ,postp
-                         ,repeat_flagp
-                         ,remaining_gapp);
+int slinke_decode(struct ir_remote *remote,
+		  ir_code          *prep,
+		  ir_code          *codep,
+		  ir_code          *postp,
+		  int              *repeat_flagp,
+		  lirc_t           *min_remaining_gapp,
+		  lirc_t           *max_remaining_gapp)
+{
+	rewind_rec_buffer(); 
+	rec_buffer.wptr = 0; 
+	signal_queue_rd_idx = 0; 
+	return receive_decode(remote,
+			      prep,
+			      codep,
+			      postp,
+			      repeat_flagp,
+			      min_remaining_gapp,
+			      max_remaining_gapp);
 } /* slinke_decode */
