@@ -1,4 +1,4 @@
-/*      $Id: hw_uirt2_common.c,v 5.6 2008/03/30 14:52:04 lirc Exp $   */
+/*      $Id: hw_uirt2_common.c,v 5.8 2010/08/27 05:08:09 jarodwilson Exp $   */
 
 /****************************************************************************
  ** hw_uirt2_common.c *******************************************************
@@ -144,8 +144,8 @@ static int mywaitfordata(uirt2_t *dev, long usec) {
 	FD_ZERO(&fds);
 	FD_SET(fd,&fds);
 
-	tv.tv_sec = 0;
-	tv.tv_usec = usec;
+	tv.tv_sec = usec / 1000000;
+	tv.tv_usec = usec % 1000000;
 
 	ret = select(maxfd + 1, &fds, NULL, NULL, &tv);
 
@@ -658,6 +658,7 @@ lirc_t uirt2_read_raw(uirt2_t *dev, lirc_t timeout)
 			dev->new_signal = 0;
 		} else {
 			data = UIRT2_UNIT * b;
+			if(data == 0) data = 1;
 			if (pulse) {
 				data = data | PULSE_BIT;
 			}
