@@ -1,4 +1,4 @@
-/*      $Id: ir_remote_types.h,v 5.12 2008/06/02 20:48:41 lirc Exp $      */
+/*      $Id: ir_remote_types.h,v 5.17 2010/04/11 18:50:38 lirc Exp $      */
 
 /****************************************************************************
  ** ir_remote_types.h *******************************************************
@@ -56,7 +56,7 @@ struct ir_ncode {
 
 /* definitions for flags */
 
-#define IR_PROTOCOL_MASK 0x03ff
+#define IR_PROTOCOL_MASK 0x07ff
 
 /* protocols: must not be combined */
 /* Don't forget to take a look at config_file.h when adding new flags */
@@ -76,19 +76,20 @@ struct ir_ncode {
 #define GRUNDIG         0x0080    /* encoding found on Grundig remote */
 #define BO              0x0100    /* encoding found on Bang & Olufsen remote */
 #define SERIAL          0x0200    /* serial protocol */
+#define XMP             0x0400    /* XMP protocol */
 
 /* additinal flags: can be orred together with protocol flag */
-#define REVERSE		0x0400
-#define NO_HEAD_REP	0x0800	  /* no header for key repeats */
-#define NO_FOOT_REP	0x1000	  /* no foot for key repeats */
-#define CONST_LENGTH    0x2000    /* signal length+gap is always constant */
-#define REPEAT_HEADER   0x4000    /* header is also sent before repeat code */
+#define REVERSE		0x0800
+#define NO_HEAD_REP	0x1000	  /* no header for key repeats */
+#define NO_FOOT_REP	0x2000	  /* no foot for key repeats */
+#define CONST_LENGTH    0x4000    /* signal length+gap is always constant */
+#define REPEAT_HEADER   0x8000    /* header is also sent before repeat code */
 
-#define COMPAT_REVERSE  0x8000    /* compatibility mode for REVERSE flag */
+#define COMPAT_REVERSE  0x00010000    /* compatibility mode for REVERSE flag */
 
 /* stop repeating after 600 signals (approx. 1 minute) */
 /* update technical.html when changing this value */
-#define REPEAT_MAX 600
+#define REPEAT_MAX_DEFAULT 600
 
 #define DEFAULT_FREQ 38000
 
@@ -143,6 +144,7 @@ struct ir_remote
 	int toggle_bit;             /* obsolete */
 	ir_code toggle_bit_mask;    /* previously only one bit called
 				       toggle_bit */
+	int suppress_repeat;        /* suppress unwanted repeats */
 	int min_repeat;             /* code is repeated at least x times
 				       code sent once -> min_repeat=0 */
 	unsigned int min_code_repeat;/*meaningful only if remote sends
@@ -178,6 +180,16 @@ struct ir_remote
 	struct timeval last_send;   /* time last_code was received or sent */
 	lirc_t min_remaining_gap;   /* remember gap for CONST_LENGTH remotes */
 	lirc_t max_remaining_gap;   /* gap range */
+	
+	lirc_t min_total_signal_length; /* how long is the shortest
+					   signal including gap */
+	lirc_t max_total_signal_length; /* how long is the longest
+					   signal including gap */
+	lirc_t min_gap_length;      /* how long is the shortest gap */
+	lirc_t max_gap_length;      /* how long is the longest gap */
+	lirc_t min_pulse_length, max_pulse_length;
+	lirc_t min_space_length, max_space_length;
+	int release_detected;       /* set by release generator */
         struct ir_remote *next;
 };
 
