@@ -1,4 +1,4 @@
-/*      $Id: lircd.c,v 5.80 2008/10/04 21:48:43 lirc Exp $      */
+/*      $Id: lircd.c,v 5.97 2010/08/27 05:08:09 jarodwilson Exp $      */
 
 /****************************************************************************
  ** lircd.c *****************************************************************
@@ -57,8 +57,14 @@
 #include <fcntl.h>
 #include <sys/file.h>
 
-#if defined __APPLE__
-#include <sys/ioccom.h>
+#if defined(__linux__)
+#include <linux/input.h>
+#include <linux/uinput.h>
+#include "input_map.h"
+#endif
+
+#if defined __APPLE__  || defined __FreeBSD__
+#include <sys/ioctl.h>
 #endif
 
 #ifndef timersub
@@ -90,11 +96,12 @@ extern struct ir_ncode *repeat_code;
 
 static int repeat_fd=-1;
 static char *repeat_message=NULL;
+static unsigned int repeat_max = REPEAT_MAX_DEFAULT;
 
 extern struct hardware hw;
 
 char *progname="lircd";
-char *configfile=LIRCDCFGFILE;
+const char *configfile = NULL;
 #ifndef USE_SYSLOG
 char *logfile=LOGFILE;
 #else
@@ -149,33 +156,46 @@ char hostname[HOSTNAME_LEN+1];
 FILE *lf=NULL;
 #endif
 
-/* quite arbitrary limits */
-#define MAX_PEERS	100
-/* substract one for lirc, sockfd, sockinet, logfile, pidfile */
-#define MAX_CLIENTS     (FD_SETSIZE-5-MAX_PEERS)
+/* substract one for lirc, sockfd, sockinet, logfile, pidfile, uinput */
+#define MAX_PEERS	((FD_SETSIZE-6)/2)
+#define MAX_CLIENTS     ((FD_SETSIZE-6)/2)
 
 int sockfd, sockinet;
+static int uinputfd = -1;
 int clis[MAX_CLIENTS];
 
 #define CT_LOCAL  1
 #define CT_REMOTE 2
 
-int cli_type[MAX_CLIENTS];
-int clin=0;
+static int cli_type[MAX_CLIENTS];
+static int clin=0;
 
 int listen_tcpip=0;
 unsigned short int port=LIRC_INET_PORT;
+struct in_addr address;
 
 struct	peer_connection *peers[MAX_PEERS];
 int	peern = 0;
 
 int debug=0;
-int daemonized=0;
-int allow_simulate=0;
+static int daemonized=0;
+static int allow_simulate=0;
+static int userelease=0;
+static int useuinput=0;
 
 static sig_atomic_t term=0,hup=0,alrm=0;
 static int termsig;
 
+static unsigned int setup_min_freq=0, setup_max_freq=0;
+static lirc_t setup_max_gap=0;
+static lirc_t setup_min_pulse=0, setup_min_space=0;
+static lirc_t setup_max_pulse=0, setup_max_space=0;
+
+inline int use_hw()
+{
+	return (clin>0 || (useuinput && uinputfd != -1) || repeat_remote != NULL);
+}
+
 /* set_transmitters only supports 32 bit int */
 #define MAX_TX (CHAR_BIT*sizeof(unsigned long))
 
@@ -303,6 +323,7 @@ void dosigterm(int sig)
 		free_config(free_remotes);
 	}
 	free_config(remotes);
+	repeat_remote = NULL;
 	logprintf(LOG_NOTICE,"caught signal");
 	for (i=0; i<clin; i++)
 	{
@@ -311,6 +332,15 @@ void dosigterm(int sig)
 	};
 	shutdown(sockfd,2);
 	close(sockfd);
+
+#if defined(__linux__)
+	if(uinputfd != -1)
+	{
+		ioctl(uinputfd, UI_DEV_DESTROY);
+		close(uinputfd);
+		uinputfd = -1;
+	}
+#endif
 	if(listen_tcpip)
 	{
 		shutdown(sockinet,2);
@@ -318,7 +348,7 @@ void dosigterm(int sig)
 	}
 	fclose(pidf);
 	(void) unlink(pidfile);
-	if(clin>0 && hw.deinit_func) hw.deinit_func();
+	if(use_hw() && hw.deinit_func) hw.deinit_func();
 #ifdef USE_SYSLOG
 	closelog();
 #else
@@ -387,10 +417,234 @@ void dosighup(int sig)
       }
 }
 
+int setup_uinputfd(const char *name)
+{
+#if defined(__linux__)
+	int fd;
+	int key;
+	struct uinput_user_dev dev;
+	
+	fd = open("/dev/input/uinput", O_RDWR);
+	if(fd == -1)
+	{
+		fd = open("/dev/uinput", O_RDWR);
+		if(fd == -1)
+		{
+			fd = open("/dev/misc/uinput", O_RDWR);
+			if(fd == -1)
+			{
+				fprintf(stderr, "could not open %s\n",
+					"uinput");
+				perror(NULL);
+				return -1;
+			}
+		}
+	}
+	memset(&dev, 0, sizeof(dev));
+	strncpy(dev.name, name, sizeof(dev.name));
+	dev.name[sizeof(dev.name)-1] = 0;
+	if(write(fd, &dev, sizeof(dev)) != sizeof(dev) ||
+	   ioctl(fd, UI_SET_EVBIT, EV_KEY) != 0 ||
+	   ioctl(fd, UI_SET_EVBIT, EV_REP) != 0)
+	{
+		goto setup_error;
+	}
+
+        for(key = KEY_RESERVED; key <= KEY_UNKNOWN; key++)
+	{
+                if(ioctl(fd, UI_SET_KEYBIT, key) != 0)
+		{
+			goto setup_error;
+                }
+	}
+
+	if(ioctl(fd, UI_DEV_CREATE) != 0)
+	{
+		goto setup_error;
+	}
+	return fd;
+	
+ setup_error:
+	fprintf(stderr, "could not setup %s\n", "uinput");
+	perror(NULL);
+	close(fd);
+#endif
+	return -1;
+}
+
+static int setup_frequency()
+{
+	unsigned int freq;
+	
+	if(!(hw.features&LIRC_CAN_SET_REC_CARRIER))
+	{
+		return(1);
+	}
+	if(setup_min_freq == 0 || setup_max_freq == 0)
+	{
+		setup_min_freq = DEFAULT_FREQ;
+		setup_max_freq = DEFAULT_FREQ;
+	}
+	if(hw.features&LIRC_CAN_SET_REC_CARRIER_RANGE &&
+	   setup_min_freq!=setup_max_freq)
+	{
+		if(hw.ioctl_func(LIRC_SET_REC_CARRIER_RANGE, &setup_min_freq)==-1)
+		{
+			logprintf(LOG_ERR,"could not set receive carrier");
+			logperror(LOG_ERR, __FUNCTION__);
+			return(0);
+		}
+		freq=setup_max_freq;
+	}
+	else
+	{
+		freq=(setup_min_freq+setup_max_freq)/2;
+	}
+	if(hw.ioctl_func(LIRC_SET_REC_CARRIER, &freq)==-1)
+	{
+		logprintf(LOG_ERR, "could not set receive carrier");
+		logperror(LOG_ERR, __FUNCTION__);
+		return(0);
+	}
+	return(1);
+}
+
+static int setup_timeout()
+{
+	lirc_t val, min_timeout, max_timeout;
+	
+	if(!(hw.features&LIRC_CAN_SET_REC_TIMEOUT))
+	{
+		return 1;
+	}
+	
+	if(setup_max_space == 0)
+	{
+		return 1;
+	}
+	if(hw.ioctl_func(LIRC_GET_MIN_TIMEOUT, &min_timeout) == -1 ||
+	   hw.ioctl_func(LIRC_GET_MAX_TIMEOUT, &max_timeout) == -1)
+	{
+		return 0;
+	}
+	if(setup_max_gap >= min_timeout && setup_max_gap <= max_timeout)
+	{
+		/* may help to detect end of signal faster */
+		val = setup_max_gap;
+	}
+	else
+	{
+		/* keep timeout to a minimum */
+		val = setup_max_space + 1;
+		if(val < min_timeout)
+		{
+			val = min_timeout;
+		}
+		else if(val > max_timeout)
+		{
+			/* maximum timeout smaller than maximum possible
+			   space, hmm */
+			val = max_timeout;
+		}
+	}
+	
+	if(hw.ioctl_func(LIRC_SET_REC_TIMEOUT, &val) == -1)
+	{
+		logprintf(LOG_ERR, "could not set timeout");
+		logperror(LOG_ERR, __FUNCTION__);
+		return 0;
+	}
+	else
+	{
+		unsigned int enable = 1;
+		hw.ioctl_func(LIRC_SET_REC_TIMEOUT_REPORTS, &enable);
+	}
+	return 1;
+}
+
+static int setup_filter()
+{
+	int ret1, ret2;
+	lirc_t min_pulse_supported, max_pulse_supported;
+	lirc_t min_space_supported, max_space_supported;
+	
+	if(!(hw.features&LIRC_CAN_SET_REC_FILTER))
+	{
+		return 1;
+	}
+	if(hw.ioctl_func(LIRC_GET_MIN_FILTER_PULSE,
+			 &min_pulse_supported) == -1 ||
+	   hw.ioctl_func(LIRC_GET_MAX_FILTER_PULSE,
+			 &max_pulse_supported) == -1 ||
+	   hw.ioctl_func(LIRC_GET_MIN_FILTER_SPACE,
+			 &min_space_supported) == -1 ||
+	   hw.ioctl_func(LIRC_GET_MAX_FILTER_SPACE,
+			 &max_space_supported) == -1)
+	{
+		logprintf(LOG_ERR, "could not get filter range");
+		logperror(LOG_ERR, __FUNCTION__);
+	}
+	
+	if(setup_min_pulse > max_pulse_supported)
+	{
+		setup_min_pulse = max_pulse_supported;
+	}
+	else if(setup_min_pulse < min_pulse_supported)
+	{
+		setup_min_pulse = 0; /* disable filtering */
+	}
+	
+	if(setup_min_space > max_space_supported)
+	{
+		setup_min_space = max_space_supported;
+	}
+	else if(setup_min_space < min_space_supported)
+	{
+		setup_min_space = 0; /* disable filtering */
+	}
+	
+	ret1 = hw.ioctl_func(LIRC_SET_REC_FILTER_PULSE, &setup_min_pulse);
+	ret2 = hw.ioctl_func(LIRC_SET_REC_FILTER_SPACE, &setup_min_space);
+	if(ret1 == -1 || ret2 == -1)
+	{
+		if(hw.ioctl_func(LIRC_SET_REC_FILTER,
+				 setup_min_pulse < setup_min_space ?
+				 &setup_min_pulse:&setup_min_space) == -1)
+		{
+			logprintf(LOG_ERR, "could not set filter");
+			logperror(LOG_ERR, __FUNCTION__);
+			return 0;
+		}
+	}
+	return 1;
+}
+
+static int setup_hardware()
+{
+	int ret = 1;
+	
+	if(hw.fd != -1 && hw.ioctl_func)
+	{
+		if((hw.features&LIRC_CAN_SET_REC_CARRIER) ||
+		   (hw.features&LIRC_CAN_SET_REC_TIMEOUT) ||
+		   (hw.features&LIRC_CAN_SET_REC_FILTER))
+		{
+			(void) hw.ioctl_func(LIRC_SETUP_START, NULL);
+			ret = setup_frequency() &&
+				setup_timeout() &&
+				setup_filter();
+			(void) hw.ioctl_func(LIRC_SETUP_END, NULL);
+		}
+	}
+	return ret;
+}
+
 void config(void)
 {
 	FILE *fd;
 	struct ir_remote *config_remotes;
+	const char *filename = configfile;
+	if(filename == NULL) filename = LIRCDCFGFILE;
 	
 	if(free_remotes!=NULL)
 	{
@@ -398,14 +652,29 @@ void config(void)
 		logprintf(LOG_ERR,"old config is still in use");
 		return;
 	}
-	fd=fopen(configfile,"r");
+	fd=fopen(filename, "r");
+	if(fd == NULL && errno == ENOENT && configfile == NULL)
+	{
+		/* try old lircd.conf location */
+		int save_errno = errno;
+		fd = fopen(LIRCDOLDCFGFILE, "r");
+		if(fd != NULL)
+		{
+			filename = LIRCDOLDCFGFILE;
+		}
+		else
+		{
+			errno = save_errno;
+		}
+	}
 	if(fd==NULL)
 	{
-		logprintf(LOG_ERR,"could not open config file '%s'",
-			  configfile);
-		logperror(LOG_ERR,NULL);
+		logprintf(LOG_ERR, "could not open config file '%s'",
+			  filename);
+		logperror(LOG_ERR, NULL);
 		return;
 	}
+	configfile = filename;
 	config_remotes=read_config(fd, configfile);
 	fclose(fd);
 	if(config_remotes==(void *) -1)
@@ -424,7 +693,14 @@ void config(void)
 		   as they could still be in use */
 		free_remotes=remotes;
 		remotes=config_remotes;
-		if(hw.config_func) (void) hw.config_func(remotes);
+		
+		
+		get_frequency_range(remotes,&setup_min_freq,&setup_max_freq);
+		get_filter_parameters(remotes,&setup_max_gap,
+				      &setup_min_pulse,&setup_min_space,
+				      &setup_max_pulse,&setup_max_space);
+		
+		setup_hardware();
 	}
 }
 
@@ -448,9 +724,7 @@ void remove_client(int fd)
 			logprintf(LOG_INFO,"removed client");
 			
 			clin--;
-			if(clin==0 &&
-			   repeat_remote==NULL &&
-			   hw.deinit_func)
+			if(!use_hw() && hw.deinit_func)
 			{
 				hw.deinit_func();
 			}
@@ -508,8 +782,8 @@ void add_client(int sock)
 	{
 		cli_type[clin]=0; /* what? */
 	}
-	clis[clin++]=fd;
-	if(clin==1 && repeat_remote==NULL)
+	clis[clin]=fd;
+	if(!use_hw())
 	{
 		if(hw.init_func)
 		{
@@ -523,8 +797,13 @@ void add_client(int sock)
 				 * have a device actually plugged
 				 * in. */
 			}
+			else
+			{
+				setup_hardware();
+			}
 		}
 	}
+	clin++;
 }
 
 int add_peer_connection(char *server)
@@ -665,8 +944,8 @@ int get_peer_message(struct peer_connection *peer)
 	if(length)
 	{
 		buffer[length]=0;
-		end=strchr(buffer,'\n');
-		if(end==NULL)
+		end=strrchr(buffer,'\n');
+		if(end == NULL || end[1] != 0)
 		{
 			logprintf(LOG_ERR,"bad send packet: \"%s\"",buffer);
 			/* remove clients that behave badly */
@@ -674,6 +953,7 @@ int get_peer_message(struct peer_connection *peer)
 		}
 		end++;	/* include the \n */
 		end[0]=0;
+		length=strlen(buffer);
 		LOGPRINTF(1,"received peer message: \"%s\"",buffer);
 		for(i=0;i<clin;i++)
 		{
@@ -799,6 +1079,10 @@ void start_server(mode_t permission,int nodaemon)
 	listen(sockfd,3);
 	nolinger(sockfd);
 
+	if(useuinput)
+	{
+		uinputfd = setup_uinputfd(progname);
+	}
 	if(listen_tcpip)
 	{
 		int enable=1;
@@ -815,7 +1099,7 @@ void start_server(mode_t permission,int nodaemon)
 		(void) setsockopt(sockinet,SOL_SOCKET,SO_REUSEADDR,
 				  &enable,sizeof(enable));
 		serv_addr_in.sin_family=AF_INET;
-		serv_addr_in.sin_addr.s_addr=htonl(INADDR_ANY);
+		serv_addr_in.sin_addr=address;
 		serv_addr_in.sin_port=htons(port);
 		
 		if(bind(sockinet,(struct sockaddr *) &serv_addr_in,
@@ -877,7 +1161,7 @@ void log_enable(int enabled)
 }
 
 #ifdef USE_SYSLOG
-void logprintf(int prio,char *format_str, ...)
+void logprintf(int prio, const char *format_str, ...)
 {
 	int save_errno = errno;
 	va_list ap;
@@ -899,7 +1183,7 @@ void logperror(int prio,const char *s)
 	else syslog(prio,"%m\n");
 }
 #else
-void logprintf(int prio,char *format_str, ...)
+void logprintf(int prio,const char *format_str, ...)
 {
 	int save_errno = errno;
 	va_list ap;  
@@ -995,7 +1279,7 @@ void dosigalrm(int sig)
 			free(repeat_message);
 			repeat_message=NULL;
 		}
-		if(clin==0 && repeat_remote==NULL && hw.deinit_func)
+		if(!use_hw() && hw.deinit_func)
 		{
 			hw.deinit_func();
 		}
@@ -1025,23 +1309,24 @@ void dosigalrm(int sig)
 		repeat_message=NULL;
 		repeat_fd=-1;
 	}
-	if(clin==0 && repeat_remote==NULL && hw.deinit_func)
+	if(!use_hw() && hw.deinit_func)
 	{
 		hw.deinit_func();
 	}
 }
 
 int parse_rc(int fd,char *message,char *arguments,struct ir_remote **remote,
-	     struct ir_ncode **code,int *reps,int n)
+	     struct ir_ncode **code, int *reps, int n, int *err)
 {
 	char *name=NULL,*command=NULL,*repeats,*end_ptr=NULL;
 
 	*remote=NULL;
 	*code=NULL;
-	if(arguments==NULL) return(1);
+	*err = 1;
+	if(arguments==NULL) goto arg_check;
 
 	name=strtok(arguments,WHITE_SPACE);
-	if(name==NULL) return(1);
+	if(name == NULL) goto arg_check;
 	*remote=get_ir_remote(remotes,name);
 	if(*remote==NULL)
 	{
@@ -1049,11 +1334,11 @@ int parse_rc(int fd,char *message,char *arguments,struct ir_remote **remote,
 				  name));
 	}
 	command=strtok(NULL,WHITE_SPACE);
-	if(command==NULL) return(1);
+	if(command == NULL) goto arg_check;
 	*code=get_code_by_name(*remote,command);
 	if(*code==NULL)
 	{
-		return(send_error(fd,message,"unknown command: \"%s\"\n",
+		return(send_error(fd,message, "unknown command: \"%s\"\n",
 				  command));
 	}
 	if(reps!=NULL)
@@ -1067,11 +1352,12 @@ int parse_rc(int fd,char *message,char *arguments,struct ir_remote **remote,
 				return(send_error(fd,message,
 						  "bad send packet\n"));
 			}
-			if (*reps>REPEAT_MAX)
+			if (*reps > repeat_max)
 			{
 				return(send_error
 				       (fd,message,
-					"too many repeats: \"%d\"\n",*reps));
+					"too many repeats: \"%d\" > \"%u\"\n",
+					*reps, repeat_max));
 			}
 		}
 		else
@@ -1083,6 +1369,7 @@ int parse_rc(int fd,char *message,char *arguments,struct ir_remote **remote,
 	{
 		return(send_error(fd,message,"bad send packet\n"));
 	}
+ arg_check:
 	if(n>0 && *remote==NULL)
 	{
 		return(send_error(fd,message,"remote missing\n"));
@@ -1091,6 +1378,7 @@ int parse_rc(int fd,char *message,char *arguments,struct ir_remote **remote,
 	{
 		return(send_error(fd,message,"code missing\n"));
 	}
+	*err = 0;
 	return(1);
 }
 
@@ -1265,8 +1553,11 @@ int list(int fd,char *message,char *arguments)
 {
 	struct ir_remote *remote;
 	struct ir_ncode *code;
+	int err;
 
-	if(parse_rc(fd,message,arguments,&remote,&code,NULL,0)==0) return(0);
+	if(parse_rc(fd, message, arguments,
+		    &remote, &code, NULL, 0, &err) == 0) return 0;
+	if(err) return 1;
 	
 	if(remote==NULL)
 	{
@@ -1413,14 +1704,15 @@ int send_core(int fd,char *message,char *arguments,int once)
 	struct ir_ncode *code;
 	struct itimerval repeat_timer;
 	int reps;
+	int err;
 	
 	if(hw.send_mode==0) return(send_error(fd,message,"hardware does not "
 					      "support sending\n"));
 	
-	if(parse_rc(fd,message,arguments,&remote,&code,
-		    once ? &reps:NULL,2)==0) return(0);
+	if(parse_rc(fd, message, arguments, &remote, &code,
+		    once ? &reps:NULL, 2, &err) == 0) return(0);
+	if(err) return 1;
 	
-	if(remote==NULL || code==NULL) return(1);
 	if(once)
 	{
 		if(repeat_remote!=NULL)
@@ -1458,7 +1750,7 @@ int send_core(int fd,char *message,char *arguments,int once)
 	else
 	{
 		/* you've been warned, now we have a limit */
-		remote->repeat_countdown=REPEAT_MAX;
+		remote->repeat_countdown = repeat_max;
 	}
 	if(remote->repeat_countdown>0 || code->next != NULL)
 	{
@@ -1501,21 +1793,33 @@ int send_stop(int fd,char *message,char *arguments)
 	struct ir_remote *remote;
 	struct ir_ncode *code;
 	struct itimerval repeat_timer;
+	int err;
 	
-	if(parse_rc(fd,message,arguments,&remote,&code,NULL,2)==0) return(0);
+	if(parse_rc(fd, message, arguments,
+		    &remote, &code, NULL, 0, &err) == 0) return 0;
+	if(err) return 1;
 	
-	if(remote==NULL || code==NULL) return(1);
-	if(repeat_remote && repeat_code &&
-	   strcasecmp(remote->name,repeat_remote->name)==0 && 
-	   strcasecmp(code->name,repeat_code->name)==0)
+	if(repeat_remote && repeat_code)
 	{
 		int done;
+		if(remote &&
+		   strcasecmp(remote->name, repeat_remote->name) != 0)
+		{
+			return(send_error(fd, message, "specified remote "
+					  "does not match\n"));
+		}
+		if(code && strcasecmp(code->name, repeat_code->name) != 0)
+		{
+			return(send_error(fd, message, "specified code "
+					  "does not match\n"));
+		}
 
-		done=REPEAT_MAX-remote->repeat_countdown;
-		if(done<remote->min_repeat)
+		done = repeat_max - repeat_remote->repeat_countdown;
+		if(done < repeat_remote->min_repeat)
 		{
 			/* we still have some repeats to do */
-			remote->repeat_countdown=remote->min_repeat-done;
+			repeat_remote->repeat_countdown =
+				repeat_remote->min_repeat - done;
 			return(send_success(fd,message));
 		}
 		repeat_timer.it_value.tv_sec=0;
@@ -1581,6 +1885,11 @@ int get_command(int fd)
 		packet_length=strlen(buffer)+1;
 
 		strcpy(backup,buffer);strcat(backup,"\n");
+		
+		/* remove DOS line endings */
+		end = strrchr(buffer, '\r');
+		if(end && end[1] == 0) *end = 0;
+		
 		directive=strtok(buffer,WHITE_SPACE);
 		if(directive==NULL)
 		{
@@ -1645,13 +1954,21 @@ void free_old_remotes()
 	struct ir_remote *scan_remotes,*found;
 	struct ir_ncode *code;
 	const char *release_event;
+	const char *release_remote_name;
+	const char *release_button_name;
 	
 	if(decoding ==free_remotes) return;
 	
-	release_event = release_map_remotes(free_remotes, remotes);
+	release_event = release_map_remotes(free_remotes, remotes,
+					    &release_remote_name,
+					    &release_button_name);
 	if(release_event != NULL)
 	{
-		broadcast_message(release_event);
+		input_message(release_event,
+			      release_remote_name,
+			      release_button_name,
+			      0,
+			      1);
 	}
 	if(last_remote!=NULL)
 	{
@@ -1741,19 +2058,67 @@ void free_old_remotes()
 	}
 }
 
+void input_message(const char *message, const char *remote_name,
+		   const char *button_name, int reps, int release)
+{
+	const char *release_message;
+	const char *release_remote_name;
+	const char *release_button_name;
+
+	release_message = check_release_event(&release_remote_name,
+					      &release_button_name);
+	if(release_message)
+	{
+		input_message(release_message, release_remote_name,
+			      release_button_name, 0, 1);
+	}
+	
+	if(!release || userelease)
+	{
+		broadcast_message(message);
+	}
+	
+#ifdef __linux__
+	if(uinputfd != -1)
+	{
+		linux_input_code input_code;
+		
+		if(reps < 2 &&
+		   get_input_code(button_name, &input_code) != -1)
+		{
+			struct input_event event;
+			
+			memset(&event, 0, sizeof(event));
+			event.type = EV_KEY;
+			event.code = input_code;
+			event.value = release ? 0 : (reps>0 ? 2:1);
+			if(write(uinputfd, &event, sizeof(event)) != sizeof(event))
+			{
+				logprintf(LOG_ERR, "writing to uinput failed");
+				logperror(LOG_ERR, NULL);
+			}
+
+			/* Need to write sync event */
+			memset(&event, 0, sizeof(event));
+			event.type = EV_SYN;
+			event.code = SYN_REPORT;
+			event.value = 0;
+			if(write(uinputfd, &event, sizeof(event)) != sizeof(event))
+			{
+				logprintf(LOG_ERR, "writing EV_SYN to uinput failed");
+				logperror(LOG_ERR, NULL);
+			}
+		}
+	}
+#endif
+}
+
 void broadcast_message(const char *message)
 {
 	int len,i;
-	const char *release_message;
-
-	release_message = check_release_event();
-	if(release_message)
-	{
-		broadcast_message(release_message);
-	}
-
+	
 	len=strlen(message);
-			
+	
 	for (i=0; i<clin; i++)
 	{
 		LOGPRINTF(1,"writing to client %d",i);
@@ -1799,7 +2164,7 @@ int waitfordata(long maxusec)
 				FD_SET(sockinet,&fds);
 				maxfd=max(maxfd,sockinet);
 			}
-			if(clin>0 && hw.rec_mode!=0 && hw.fd!=-1)
+			if(use_hw() && hw.rec_mode!=0 && hw.fd!=-1)
 			{
 				FD_SET(hw.fd,&fds);
 				maxfd=max(maxfd,hw.fd);
@@ -1859,7 +2224,7 @@ int waitfordata(long maxusec)
 				tv.tv_sec= maxusec / 1000000;
 				tv.tv_usec=maxusec % 1000000;
 			}
-			if(hw.fd == -1)
+			if(hw.fd == -1 && use_hw())
 			{
 				/* try to reconnect */
 				timerclear(&timeout);
@@ -1884,7 +2249,8 @@ int waitfordata(long maxusec)
 					struct timeval gap;
 					
 					timersub(&release_time, &now, &gap);
-					if(timercmp(&tv, &gap, >))
+					if(!(timerisset(&tv) || reconnect) ||
+					   timercmp(&tv, &gap, >))
 					{
 						tv = gap;
 					}
@@ -1915,10 +2281,19 @@ int waitfordata(long maxusec)
 			   timercmp(&now, &release_time, >))
 			{
 				const char *release_message;
-				release_message = trigger_release_event();
+				const char *release_remote_name;
+				const char *release_button_name;
+				
+				release_message = trigger_release_event
+					(&release_remote_name,
+					 &release_button_name);
 				if(release_message)
 				{
-					broadcast_message(release_message);
+					input_message(release_message,
+						      release_remote_name,
+						      release_button_name,
+						      0,
+						      1);
 				}
 			}
 			if(free_remotes!=NULL)
@@ -1948,10 +2323,13 @@ int waitfordata(long maxusec)
 		}
 		while(ret==-1 && errno==EINTR);
 		
-		if(hw.fd == -1 && clin > 0 && hw.init_func)
+		if(hw.fd == -1 &&
+		   use_hw() &&
+		   hw.init_func)
 		{
 			log_enable(0);
 			hw.init_func();
+			setup_hardware();
 			log_enable(1);
 		}
 		for(i=0;i<clin;i++)
@@ -1993,7 +2371,7 @@ int waitfordata(long maxusec)
 			LOGPRINTF(1,"registering inet client");
 			add_client(sockinet);
 		}
-                if(clin>0 && hw.rec_mode!=0 && hw.fd!=-1 &&
+                if(use_hw() && hw.rec_mode!=0 && hw.fd!=-1 &&
 		   FD_ISSET(hw.fd,&fds))
                 {
 			register_input();
@@ -2007,7 +2385,7 @@ void loop()
 {
 	char *message;
 	
-	logprintf(LOG_NOTICE,"lircd(%s) ready", hw.name);
+	logprintf(LOG_NOTICE,"lircd(%s) ready, using %s", hw.name, lircdfile);
 	while(1)
 	{
 		(void) waitfordata(0);
@@ -2016,13 +2394,20 @@ void loop()
 		
 		if(message!=NULL)
 		{
+			const char *remote_name;
+			const char *button_name;
+			int reps;
+			
 			if(hw.ioctl_func &&
 			   (hw.features&LIRC_CAN_NOTIFY_DECODE))
 			{
 				hw.ioctl_func(LIRC_NOTIFY_DECODE, NULL);
 			}
 			
-			broadcast_message(message);
+			get_release_data(&remote_name, &button_name, &reps);
+
+			input_message(message, remote_name, button_name,
+				      reps, 0);
 		}
 	}
 }
@@ -2034,6 +2419,7 @@ int main(int argc,char **argv)
 	mode_t permission=S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH;
 	char *device=NULL;
 
+	address.s_addr = htonl(INADDR_ANY);
 	hw_choose_driver(NULL);
 	while(1)
 	{
@@ -2058,9 +2444,16 @@ int main(int argc,char **argv)
 #                       endif
 			{"release",optional_argument,NULL,'r'},
 			{"allow-simulate",no_argument,NULL,'a'},
+#                       if defined(__linux__)
+			{"uinput",no_argument,NULL,'u'},
+#                       endif
+			{"repeat-max",required_argument,NULL,'R'},
 			{0, 0, 0, 0}
 		};
-		c = getopt_long(argc,argv,"hvnp:H:d:o:P:l::c:r::a"
+		c = getopt_long(argc,argv,"hvnp:H:d:o:P:l::c:r::aR:"
+#                               if defined(__linux__)
+				"u"
+#                               endif
 #                               ifndef USE_SYSLOG
 				"L:"
 #                               endif
@@ -2080,7 +2473,7 @@ int main(int argc,char **argv)
 			printf("\t -p --permission=mode\t\tfile permissions for " LIRCD "\n");
 			printf("\t -H --driver=driver\t\tuse given driver\n");
 			printf("\t -d --device=device\t\tread from given device\n");
-			printf("\t -l --listen[=port]\t\tlisten for network connections on port\n");
+			printf("\t -l --listen[=[address:]port]\tlisten for network connections\n");
 			printf("\t -c --connect=host[:port]\tconnect to remote lircd server\n");
                         printf("\t -o --output=socket\t\toutput socket filename\n");
                         printf("\t -P --pidfile=file\t\tdaemon pid file\n");
@@ -2092,6 +2485,10 @@ int main(int argc,char **argv)
 #                       endif
 			printf("\t -r --release[=suffix]\t\tauto-generate release events\n");
 			printf("\t -a --allow-simulate\t\taccept SIMULATE command\n");
+#                       if defined(__linux__)
+			printf("\t -u --uinput\t\tgenerate Linux input events\n");
+#                       endif
+			printf("\t -R --repeat-max=limit\t\tallow at most this many repeats\n");
 			return(EXIT_SUCCESS);
 		case 'v':
 			printf("%s %s\n",progname,VERSION);
@@ -2135,16 +2532,28 @@ int main(int argc,char **argv)
 			{
 				long p;
 				char *endptr;
-				
-				p=strtol(optarg,&endptr,10);
+				char *sep = strchr(optarg, ':');
+				char *port_str = sep ? sep + 1 : optarg;
+				p=strtol(port_str,&endptr,10);
 				if(!*optarg || *endptr || p<1 || p>USHRT_MAX)
 				{
 					fprintf(stderr,
 						"%s: bad port number \"%s\"\n",
-						progname,optarg);
+						progname,port_str);
 					return(EXIT_FAILURE);
 				}
 				port=(unsigned short int) p;
+				if (sep)
+				{
+					*sep = 0;
+					if(!inet_aton(optarg, &address))
+					{
+						fprintf(stderr,
+							"%s: bad address \"%s\"\n",
+       							progname,optarg);
+						return(EXIT_FAILURE);
+					}
+				}
 			}
 			else
 			{
@@ -2174,10 +2583,19 @@ int main(int argc,char **argv)
 			{
 				set_release_suffix(LIRC_RELEASE_SUFFIX);
 			}
+			userelease=1;
 			break;
 		case 'a':
 			allow_simulate=1;
 			break;
+#               if defined(__linux__)
+		case 'u':
+			useuinput=1;
+			break;
+#               endif
+		case 'R':
+			repeat_max = atoi(optarg);
+			break;
 		default:
 			printf("Usage: %s [options] [config-file]\n",progname);
 			return(EXIT_FAILURE);
