@@ -1,4 +1,4 @@
-/*      $Id: receive.c,v 5.35 2007/07/29 18:20:12 lirc Exp $      */
+/*      $Id: receive.c,v 5.43 2010/04/13 16:28:10 lirc Exp $      */
 
 /****************************************************************************
  ** receive.c ***************************************************************
@@ -42,7 +42,7 @@ inline void set_pending_space(lirc_t deltas)
 	rec_buffer.pendings=deltas;
 }
 
-lirc_t get_next_rec_buffer(lirc_t maxusec)
+static lirc_t get_next_rec_buffer_internal(lirc_t maxusec)
 {
 	if(rec_buffer.rptr<rec_buffer.wptr)
 	{
@@ -59,12 +59,22 @@ lirc_t get_next_rec_buffer(lirc_t maxusec)
 		{
 			lirc_t data;
 			
-			data=hw.readdata(2*maxusec<100000 ? 100000:2*maxusec);
+			data=hw.readdata(maxusec);
 			if(!data)
 			{
 				LOGPRINTF(3,"timeout: %u", maxusec);
 				return 0;
 			}
+			if(LIRC_IS_TIMEOUT(data))
+			{
+				LOGPRINTF(1,"timeout received: %lu",
+					  (unsigned long) LIRC_VALUE(data));
+				if(LIRC_VALUE(data) < maxusec)
+				{
+					return get_next_rec_buffer_internal(maxusec - LIRC_VALUE(data));
+				}
+				return 0;
+			}
 
                         rec_buffer.data[rec_buffer.wptr]=data;
                         if(rec_buffer.data[rec_buffer.wptr]==0) return(0);
@@ -88,6 +98,11 @@ lirc_t get_next_rec_buffer(lirc_t maxusec)
 	return(0);
 }
 
+lirc_t get_next_rec_buffer(lirc_t maxusec)
+{
+	return get_next_rec_buffer_internal(receive_timeout(maxusec));
+}
+
 void init_rec_buffer(void)
 {
 	memset(&rec_buffer,0,sizeof(rec_buffer));
@@ -126,18 +141,6 @@ int clear_rec_buffer(void)
 			((ir_code) buffer[i]);
 		}
 	}
-	else if(hw.rec_mode==LIRC_MODE_CODE)
-	{
-		unsigned char c;
-		
-		if(read(hw.fd,&c,1)!=1)
-		{
-			logprintf(LOG_ERR,"reading in mode LIRC_MODE_CODE "
-				  "failed");
-			return(0);
-		}
-		rec_buffer.decoded=(ir_code) c;
-	}
 	else
 	{
 		lirc_t data;
@@ -621,6 +624,7 @@ ir_code get_data(struct ir_remote *remote,int bits,int done)
 			logprintf(LOG_ERR,"invalid bit number.");
 			return((ir_code) -1);
 		}
+		if(!sync_pending_space(remote)) return 0;
 		for(i=0;i<bits;i+=2)
 		{
 			code<<=2;
@@ -969,6 +973,44 @@ ir_code get_data(struct ir_remote *remote,int bits,int done)
 		}
 		return code;
 	}
+	else if(is_xmp(remote))
+	{
+		lirc_t deltap,deltas,sum;
+		ir_code n;
+		
+		if(bits%4 || done%4)
+		{
+			logprintf(LOG_ERR,"invalid bit number.");
+			return((ir_code) -1);
+		}
+		if(!sync_pending_space(remote)) return 0;
+		for(i=0;i<bits;i+=4)
+		{
+			code<<=4;
+			deltap=get_next_pulse(remote->pzero);
+			deltas=get_next_space(remote->szero+16*remote->sone);
+			if(deltap==0 || deltas==0) 
+			{
+				logprintf(LOG_ERR,"failed on bit %d",
+					  done+i+1);
+				return((ir_code) -1);
+			}
+			sum=deltap+deltas;
+			
+			sum -= remote->pzero + remote->szero;
+			n = (sum + remote->sone/2)/remote->sone;
+			if(n >= 16)
+			{
+				logprintf(LOG_ERR,"failed on bit %d",
+					  done+i+1);
+				return((ir_code) -1);
+			}
+			LOGPRINTF(1, "%d: %lx", i, n);
+			code |= n;
+
+		}
+		return code;
+	}
 	
 	for(i=0;i<bits;i++)
 	{
@@ -1055,7 +1097,7 @@ int receive_decode(struct ir_remote *remote,
 		   int *repeat_flagp,
 		   lirc_t *min_remaining_gapp, lirc_t *max_remaining_gapp)
 {
-	ir_code pre,code,post,code_mask=0,post_mask=0;
+	ir_code pre,code,post;
 	lirc_t sync;
 	int header;
 	struct timeval current;
@@ -1152,8 +1194,7 @@ int receive_decode(struct ir_remote *remote,
 		struct ir_ncode *codes,*found;
 		int i;
 
-		if(hw.rec_mode==LIRC_MODE_CODE ||
-		   hw.rec_mode==LIRC_MODE_LIRCCODE)
+		if(hw.rec_mode==LIRC_MODE_LIRCCODE)
 			return(0);
 
 		codes=remote->codes;
@@ -1180,55 +1221,41 @@ int receive_decode(struct ir_remote *remote,
 				}
 			}
 			codes++;
-		}
-		if(found!=NULL)
-		{
-			if(!get_gap(remote,
-				    is_const(remote) ? 
-				    min_gap(remote)-rec_buffer.sum:
-				    min_gap(remote))) 
-				found=NULL;
+			if(found!=NULL)
+			{
+				if(!get_gap(remote,
+					    is_const(remote) ? 
+					    min_gap(remote)-rec_buffer.sum:
+					    min_gap(remote))) 
+					found=NULL;
+			}
 		}
 		if(found==NULL) return(0);
 		code=found->code;
 	}
 	else
 	{
-		if(hw.rec_mode==LIRC_MODE_CODE ||
-		   hw.rec_mode==LIRC_MODE_LIRCCODE)
+		if(hw.rec_mode==LIRC_MODE_LIRCCODE)
 		{
-			int i;
  			lirc_t sum;
+			ir_code decoded = rec_buffer.decoded;
 
 #                       ifdef LONG_IR_CODE
-			LOGPRINTF(1,"decoded: %llx",rec_buffer.decoded);
+			LOGPRINTF(1,"decoded: %llx", decoded);
 #                       else
-			LOGPRINTF(1,"decoded: %lx",rec_buffer.decoded);
+			LOGPRINTF(1,"decoded: %lx", decoded);
 #                       endif
-			if((hw.rec_mode==LIRC_MODE_CODE &&
-			    hw.code_length<bit_count(remote))
-			   ||
-			   (hw.rec_mode==LIRC_MODE_LIRCCODE && 
-			    hw.code_length!=bit_count(remote)))
+			if(hw.rec_mode==LIRC_MODE_LIRCCODE && 
+			   hw.code_length!=bit_count(remote))
 			{
 				return(0);
 			}
 			
-			for(i=0;i<remote->post_data_bits;i++)
-			{
-				post_mask=(post_mask<<1)+1;
-			}
-			post=rec_buffer.decoded&post_mask;
-			post_mask=0;
-			rec_buffer.decoded=
-			rec_buffer.decoded>>remote->post_data_bits;
-			for(i=0;i<remote->bits;i++)
-			{
-				code_mask=(code_mask<<1)+1;
-			}
-			code=rec_buffer.decoded&code_mask;
-			code_mask=0;
-			pre=rec_buffer.decoded>>remote->bits;
+			post = decoded & gen_mask(remote->post_data_bits);
+			decoded >>= remote->post_data_bits;
+			code = decoded & gen_mask(remote->bits);
+			pre = decoded >> remote->bits;
+			
 			gettimeofday(&current,NULL);
 			sum=remote->phead+remote->shead+
 				lirc_t_max(remote->pone+remote->sone,
@@ -1337,8 +1364,7 @@ int receive_decode(struct ir_remote *remote,
 		*repeat_flagp=1;
 	else
 		*repeat_flagp=0;
-	if(hw.rec_mode==LIRC_MODE_CODE ||
-	   hw.rec_mode==LIRC_MODE_LIRCCODE)
+	if(hw.rec_mode==LIRC_MODE_LIRCCODE)
 	{
 		/* Most TV cards don't pass each signal to the
                    driver. This heuristic should fix repeat in such
