@@ -1,4 +1,4 @@
-/*      $Id: serial.c,v 5.14 2007/04/23 16:25:11 lirc Exp $      */
+/*      $Id: serial.c,v 5.16 2010/08/27 05:07:00 jarodwilson Exp $      */
 
 /****************************************************************************
  ** serial.c ****************************************************************
@@ -85,6 +85,7 @@ int tty_setdtr(int fd, int enable)
 {
         int cmd, sts;
 
+#ifdef DEBUG
         if (ioctl(fd, TIOCMGET, &sts) < 0)
         {
                 LOGPRINTF(1,"%s: ioctl(TIOCMGET) failed", __FUNCTION__);
@@ -99,6 +100,7 @@ int tty_setdtr(int fd, int enable)
         {
                 LOGPRINTF(1, "%s: 1->0", __FUNCTION__);
         }
+#endif
         if (enable)
         {
                 cmd = TIOCMBIS;
@@ -526,6 +528,11 @@ int tty_delete_lock(void)
 	{
 		while((ep=readdir(dp)))
 		{
+			if (strcmp(ep->d_name,".") == 0 || strcmp(ep->d_name,"..") == 0)
+			{
+				retval = 0;
+				continue;
+			}
 			strcpy(filename,"/var/lock/");
 			if(strlen(filename)+strlen(ep->d_name)>FILENAME_MAX) 
 			{retval=0;continue;}
@@ -537,7 +544,7 @@ int tty_delete_lock(void)
 			if(len<=0) {retval=0;continue;}
 			id[len]=0;
 			pid=strtol(id,&endptr,10);
-			if(!*id || *endptr!='\n')
+			if(!*id || (*endptr && *endptr!='\n'))
 			{
 				logprintf(LOG_WARNING,"invalid lockfile (%s) "
 					  "detected",filename);
