@@ -2,7 +2,7 @@
  *   lirc_imon.c:  LIRC/VFD/LCD driver for SoundGraph iMON IR/VFD/LCD
  *		   including the iMON PAD model
  *
- *   $Id: lirc_imon.c,v 1.111 2009/09/11 04:56:18 jarodwilson Exp $
+ *   $Id: lirc_imon.c,v 1.119 2010/07/11 02:59:43 jarodwilson Exp $
  *
  *   Copyright(C) 2004  Venky Raju(dev@venky.ws)
  *
@@ -30,7 +30,9 @@
 #error "*** Sorry, this driver requires a 2.6 kernel"
 #endif
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 33)
 #include <linux/autoconf.h>
+#endif
 
 #include <linux/errno.h>
 #include <linux/init.h>
@@ -250,7 +252,7 @@ static struct usb_device_id imon_usb_id_table[] = {
 	{ USB_DEVICE(0x15c2, 0x0037) },
 	/* SoundGraph iMON OEM LCD (IR & LCD) */
 	{ USB_DEVICE(0x15c2, 0x0038) },
-	/* device specifics unknown */
+	/* SoundGraph iMON UltraBay (IR & LCD) */
 	{ USB_DEVICE(0x15c2, 0x0039) },
 	/* device specifics unknown */
 	{ USB_DEVICE(0x15c2, 0x003a) },
@@ -318,6 +320,7 @@ static struct usb_device_id ctl_ep_device_list[] = {
 static struct usb_device_id lcd_device_list[] = {
 	{ USB_DEVICE(0x15c2, 0xffdc) },
 	{ USB_DEVICE(0x15c2, 0x0038) },
+	{ USB_DEVICE(0x15c2, 0x0039) },
 	{ USB_DEVICE(0x15c2, 0x0045) },
 	{}
 };
@@ -327,6 +330,7 @@ static struct usb_device_id large_buffer_list[] = {
 	{ USB_DEVICE(0x15c2, 0x0034) },
 	{ USB_DEVICE(0x15c2, 0x0035) },
 	{ USB_DEVICE(0x15c2, 0x0038) },
+	{ USB_DEVICE(0x15c2, 0x0039) },
 	{ USB_DEVICE(0x15c2, 0x0045) },
 };
 
@@ -565,17 +569,23 @@ static int display_close(struct inode *inode, struct file *file)
 }
 
 /**
- * Sends a packet to the device
+ * Sends a packet to the device -- this function must be called
+ * with context->lock held.
  */
 static int send_packet(struct imon_context *context)
 {
 	unsigned int pipe;
+	unsigned long timeout;
 	int interval = 0;
 	int retval = 0;
 	struct usb_ctrlrequest *control_req = NULL;
 
 	/* Check if we need to use control or interrupt urb */
 	if (!context->tx_control) {
+		if (!context->tx_endpoint) {
+			err("%s: device has no tx endpoint", __func__);
+			return -EINVAL;
+		}
 		pipe = usb_sndintpipe(context->usbdev_intf0,
 				      context->tx_endpoint->bEndpointAddress);
 		interval = context->tx_endpoint->bInterval;
@@ -635,6 +645,15 @@ static int send_packet(struct imon_context *context)
 
 	kfree(control_req);
 
+	/*
+	 * Induce a mandatory 5ms delay before returning, as otherwise,
+	 * send_packet can get called so rapidly as to overwhelm the device,
+	 * particularly on faster systems and/or those with quirky usb.
+	 */
+	timeout = msecs_to_jiffies(5);
+	set_current_state(TASK_UNINTERRUPTIBLE);
+	schedule_timeout(timeout);
+
 	return retval;
 }
 
@@ -743,7 +762,6 @@ static int send_set_imon_clock(struct imon_context *context,
 		return -ENODEV;
 	}
 
-
 	for (i = 0; i < IMON_CLOCK_ENABLE_PACKETS; i++) {
 		memcpy(context->usb_tx_buf, clock_enable_pkt[i], 8);
 		retval = send_packet(context);
@@ -755,7 +773,6 @@ static int send_set_imon_clock(struct imon_context *context,
 	}
 
 	return retval;
-
 }
 
 /**
@@ -1097,13 +1114,6 @@ static void imon_set_ir_protocol(struct imon_context *context)
 
 	switch (ir_protocol) {
 	case IMON_IR_PROTOCOL_MCE:
-		/* MCE proto not supported on devices without tx control */
-		if (!context->tx_control) {
-			printk(KERN_INFO "%s: MCE IR protocol not supported on "
-			       "this device, using iMON protocol\n", __func__);
-			context->ir_protocol = IMON_IR_PROTOCOL_IMON;
-			return;
-		}
 		dprintk("Configuring IR receiver for MCE protocol\n");
 		ir_proto_packet[0] = 0x01;
 		context->ir_protocol = IMON_IR_PROTOCOL_MCE;
@@ -1125,11 +1135,18 @@ static void imon_set_ir_protocol(struct imon_context *context)
 		context->ir_protocol = IMON_IR_PROTOCOL_IMON;
 		break;
 	}
-	memcpy(context->usb_tx_buf, &ir_proto_packet,
-	       sizeof(ir_proto_packet));
+
+	memcpy(context->usb_tx_buf, &ir_proto_packet, sizeof(ir_proto_packet));
+
 	retval = send_packet(context);
-	if (retval)
-		printk(KERN_INFO "%s: failed to set remote type\n", __func__);
+	if (retval) {
+		printk(KERN_INFO "%s: failed to set IR protocol, falling back "
+		       "to standard iMON protocol mode\n", __func__);
+		ir_protocol = IMON_IR_PROTOCOL_IMON;
+		context->ir_protocol = IMON_IR_PROTOCOL_IMON;
+	}
+
+	return;
 }
 
 
@@ -1152,8 +1169,11 @@ static int ir_open(void *data)
 	context->rx.prev_bit = 0;
 
 	/* set new IR protocol if it has changed since init or last open */
-	if (ir_protocol != context->ir_protocol)
+	if (ir_protocol != context->ir_protocol) {
+		mutex_lock(&context->lock);
 		imon_set_ir_protocol(context);
+		mutex_unlock(&context->lock);
+	}
 
 	context->ir_isopen = 1;
 	printk(KERN_INFO MOD_NAME ": IR port opened\n");
@@ -1353,6 +1373,9 @@ static void imon_incoming_packet(struct imon_context *context,
 	const unsigned char ch_down[] = { 0x28, 0x87, 0x95, 0xb7 };
 #endif
 
+	if (!context->ir_onboard_decode && context->ir_isopen)
+		goto handle_rawir;
+
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 18)
 	mouse = context->mouse;
 	if (context->display_type == IMON_DISPLAY_TYPE_VGA)
@@ -1365,11 +1388,11 @@ static void imon_incoming_packet(struct imon_context *context,
 			context->pad_mouse = ~(context->pad_mouse) & 0x1;
 			dprintk("toggling to %s mode\n",
 				context->pad_mouse ? "mouse" : "keyboard");
+			return;
 		} else {
 			context->pad_mouse = 0;
-			dprintk("mouse mode was disabled by modparam\n");
+			dprintk("mouse mode disabled, passing key value\n");
 		}
-		return;
 	}
 
 	/* send touchscreen events through input subsystem if touchpad data */
@@ -1599,6 +1622,8 @@ static void imon_incoming_packet(struct imon_context *context,
 	     (buf[6] == 0x5E && buf[7] == 0xAF)))	/* DT */
 		return;		/* filler frame, no data here */
 
+handle_rawir:
+
 	if (debug) {
 		if (context->ir_onboard_decode)
 			printk("intf%d decoded packet: ", intf);
@@ -2199,7 +2224,8 @@ static int imon_probe(struct usb_interface *interface,
 	}
 
 	/* set IR protocol/remote type */
-	imon_set_ir_protocol(context);
+	if (context->tx_control || context->tx_endpoint)
+		imon_set_ir_protocol(context);
 
 	printk(KERN_INFO MOD_NAME ": iMON device (%04x:%04x, intf%d) on "
 	       "usb<%d:%d> initialized\n", vendor, product, ifnum,
